"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const utils_1 = require("./utils");
const ModelsSerializer_1 = tslib_1.__importDefault(require("./builders/ModelsSerializer"));
const JsonDeserializer_1 = tslib_1.__importDefault(require("./builders/JsonDeserializer"));
const ReduxObjectDenormalizer_1 = tslib_1.__importDefault(require("./builders/ReduxObjectDenormalizer"));
const simplePropertyMappers_1 = require("./simplePropertyMappers");
const cache_1 = require("./cache");
class Jsona {
    modelPropertiesMapper = new simplePropertyMappers_1.ModelPropertiesMapper();
    jsonPropertiesMapper = new simplePropertyMappers_1.JsonPropertiesMapper();
    DeserializeCache = cache_1.DeserializeCache;
    ModelsSerializer = ModelsSerializer_1.default;
    JsonDeserializer = JsonDeserializer_1.default;
    constructor(params) {
        if (params && params.modelPropertiesMapper) {
            this.modelPropertiesMapper = params.modelPropertiesMapper;
        }
        if (params && params.jsonPropertiesMapper) {
            this.jsonPropertiesMapper = params.jsonPropertiesMapper;
        }
        if (params && params.DeserializeCache) {
            this.DeserializeCache = params.DeserializeCache;
        }
        if (params && params.ModelsSerializer) {
            this.ModelsSerializer = params.ModelsSerializer;
        }
        if (params && params.JsonDeserializer) {
            this.JsonDeserializer = params.JsonDeserializer;
        }
    }
    /**
     * serialize
     * Creates JSON, compatible with json:api specification from Jsona model(s).
     */
    serialize({ stuff, includeNames }) {
        if (!stuff) {
            throw new Error('Jsona can not serialize, stuff is not passed');
        }
        const jsonBuilder = new this.ModelsSerializer(this.modelPropertiesMapper);
        jsonBuilder.setStuff(stuff);
        if (includeNames) {
            jsonBuilder.setIncludeNames(includeNames);
        }
        return jsonBuilder.build();
    }
    /**
     * deserialize
     * Creates Jsona model(s) from JSON, compatible with json:api specification.
     */
    deserialize(body, options) {
        if (!body) {
            throw new Error('Jsona can not deserialize, body is not passed');
        }
        const deserializeCache = new this.DeserializeCache();
        const modelBuilder = new this.JsonDeserializer(this.jsonPropertiesMapper, deserializeCache, options);
        if (typeof body === 'string') {
            modelBuilder.setJsonParsedObject((0, utils_1.jsonParse)(body));
        }
        else {
            modelBuilder.setJsonParsedObject(body);
        }
        return modelBuilder.build();
    }
    /**
     * denormalizeReduxObject
     * Creates Jsona model(s) from ReduxObject, that creates by json-api-normalizer
     * https://github.com/yury-dymov/json-api-normalizer
     *
     */
    denormalizeReduxObject({ reduxObject, entityType, entityIds, returnBuilderInRelations = false }) {
        if (!reduxObject) {
            throw new Error('Jsona can not denormalize ReduxObject, incorrect reduxObject passed');
        }
        if (!entityType) {
            throw new Error('Jsona can not denormalize ReduxObject, entityType is not passed');
        }
        if (!reduxObject[entityType]) {
            return null;
        }
        const modelBuilder = new ReduxObjectDenormalizer_1.default(this.jsonPropertiesMapper);
        modelBuilder.setReduxObject(reduxObject);
        modelBuilder.setEntityType(entityType);
        modelBuilder.setReturnBuilderInRelations(returnBuilderInRelations);
        if (entityIds) {
            modelBuilder.setEntityIds(Array.isArray(entityIds) ? entityIds : entityIds.toString());
        }
        return modelBuilder.build();
    }
}
exports.default = Jsona;
//# sourceMappingURL=Jsona.js.map