"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonPropertiesMapper = exports.defineRelationGetter = exports.ModelPropertiesMapper = exports.RELATIONSHIP_NAMES_PROP = void 0;
exports.RELATIONSHIP_NAMES_PROP = 'relationshipNames';
class ModelPropertiesMapper {
    getId(model) {
        return model.id;
    }
    getType(model) {
        return model.type;
    }
    getAttributes(model) {
        let exceptProps = ['id', 'type', exports.RELATIONSHIP_NAMES_PROP];
        if (Array.isArray(model[exports.RELATIONSHIP_NAMES_PROP])) {
            exceptProps.push(...model[exports.RELATIONSHIP_NAMES_PROP]);
        }
        else if (model[exports.RELATIONSHIP_NAMES_PROP]) {
            console.warn(`Can't getAttributes correctly, '${exports.RELATIONSHIP_NAMES_PROP}' property of ${model.type}-${model.id} model
                isn't array of relationship names`, model[exports.RELATIONSHIP_NAMES_PROP]);
        }
        const attributes = {};
        Object.keys(model).forEach((attrName) => {
            if (exceptProps.indexOf(attrName) === -1) {
                attributes[attrName] = model[attrName];
            }
        });
        return attributes;
    }
    getRelationships(model) {
        const relationshipNames = model[exports.RELATIONSHIP_NAMES_PROP];
        if (!relationshipNames || Array.isArray(relationshipNames) && !relationshipNames.length) {
            return;
        }
        else if (relationshipNames && !Array.isArray(relationshipNames)) {
            console.warn(`Can't getRelationships correctly,
                '${exports.RELATIONSHIP_NAMES_PROP}' property of ${model.type}-${model.id} model
                isn't array of relationship names`, model[exports.RELATIONSHIP_NAMES_PROP]);
            return;
        }
        const relationships = {};
        relationshipNames.forEach((relationName) => {
            if (model[relationName] !== undefined) {
                relationships[relationName] = model[relationName];
            }
        });
        return relationships;
    }
}
exports.ModelPropertiesMapper = ModelPropertiesMapper;
function defineRelationGetter(model, relationName, buildRelation) {
    Object.defineProperty(model, relationName, {
        enumerable: true,
        configurable: true,
        set: (value) => {
            delete model[relationName];
            model[relationName] = value;
        },
        get: () => {
            delete model[relationName];
            return model[relationName] = buildRelation();
        },
    });
}
exports.defineRelationGetter = defineRelationGetter;
class JsonPropertiesMapper {
    createModel(type) {
        return { type };
    }
    setId(model, id) {
        model.id = id;
    }
    setAttributes(model, attributes) {
        Object.keys(attributes).forEach((propName) => {
            model[propName] = attributes[propName];
        });
    }
    setMeta(model, meta) {
        model.meta = meta;
    }
    setLinks(model, links) {
        model.links = links;
    }
    setResourceIdObjMeta(model, meta) {
        model.resourceIdObjMeta = meta;
    }
    setRelationships(model, relationships) {
        Object.keys(relationships).forEach((propName) => {
            if (typeof relationships[propName] === 'function') {
                defineRelationGetter(model, propName, relationships[propName]);
            }
            else {
                model[propName] = relationships[propName];
            }
        });
        const newNames = Object.keys(relationships);
        const currentNames = model[exports.RELATIONSHIP_NAMES_PROP];
        if (currentNames && currentNames.length) {
            model[exports.RELATIONSHIP_NAMES_PROP] = [...currentNames, ...newNames].filter((value, i, self) => self.indexOf(value) === i);
        }
        else {
            model[exports.RELATIONSHIP_NAMES_PROP] = newNames;
        }
    }
    setRelationshipLinks(parentModel, relationName, links) {
        // inherit your IJsonPropertiesMapper and overload this method, if you want to handle relationship links
    }
    setRelationshipMeta(parentModel, relationName, links) {
        // inherit your IJsonPropertiesMapper and overload this method, if you want to handle relationship meta
    }
}
exports.JsonPropertiesMapper = JsonPropertiesMapper;
//# sourceMappingURL=simplePropertyMappers.js.map